from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
DESCRIPTOR = descriptor.FileDescriptor(name='Math.proto', package='EA.Sims4.Network', serialized_pb='\n\nMath.proto\x12\x10EA.Sims4.Network"\x1f\n\x07Vector2\x12\t\n\x01x\x18\x01 \x02(\x02\x12\t\n\x01y\x18\x02 \x02(\x02"*\n\x07Vector3\x12\t\n\x01x\x18\x01 \x02(\x02\x12\t\n\x01y\x18\x02 \x02(\x02\x12\t\n\x01z\x18\x03 \x02(\x02"8\n\nQuaternion\x12\t\n\x01x\x18\x01 \x02(\x02\x12\t\n\x01y\x18\x02 \x02(\x02\x12\t\n\x01z\x18\x03 \x02(\x02\x12\t\n\x01w\x18\x04 \x02(\x02"n\n\tTransform\x12.\n\x0btranslation\x18\x01 \x01(\x0b2\x19.EA.Sims4.Network.Vector3\x121\n\x0borientation\x18\x02 \x01(\x0b2\x1c.EA.Sims4.Network.Quaternion"k\n\x0bLinearCurve\x128\n\x06points\x18\x01 \x03(\x0b2(.EA.Sims4.Network.LinearCurve.CurvePoint\x1a"\n\nCurvePoint\x12\t\n\x01x\x18\x01 \x02(\x02\x12\t\n\x01y\x18\x02 \x02(\x02')
_VECTOR2 = descriptor.Descriptor(name='Vector2', full_name='EA.Sims4.Network.Vector2', filename=None, file=DESCRIPTOR, containing_type=None, fields=[descriptor.FieldDescriptor(name='x', full_name='EA.Sims4.Network.Vector2.x', index=0, number=1, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None), descriptor.FieldDescriptor(name='y', full_name='EA.Sims4.Network.Vector2.y', index=1, number=2, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None)], extensions=[], nested_types=[], enum_types=[], options=None, is_extendable=False, extension_ranges=[], serialized_start=32, serialized_end=63)
_VECTOR3 = descriptor.Descriptor(name='Vector3', full_name='EA.Sims4.Network.Vector3', filename=None, file=DESCRIPTOR, containing_type=None, fields=[descriptor.FieldDescriptor(name='x', full_name='EA.Sims4.Network.Vector3.x', index=0, number=1, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None), descriptor.FieldDescriptor(name='y', full_name='EA.Sims4.Network.Vector3.y', index=1, number=2, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None), descriptor.FieldDescriptor(name='z', full_name='EA.Sims4.Network.Vector3.z', index=2, number=3, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None)], extensions=[], nested_types=[], enum_types=[], options=None, is_extendable=False, extension_ranges=[], serialized_start=65, serialized_end=107)
_QUATERNION = descriptor.Descriptor(name='Quaternion', full_name='EA.Sims4.Network.Quaternion', filename=None, file=DESCRIPTOR, containing_type=None, fields=[descriptor.FieldDescriptor(name='x', full_name='EA.Sims4.Network.Quaternion.x', index=0, number=1, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None), descriptor.FieldDescriptor(name='y', full_name='EA.Sims4.Network.Quaternion.y', index=1, number=2, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None), descriptor.FieldDescriptor(name='z', full_name='EA.Sims4.Network.Quaternion.z', index=2, number=3, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None), descriptor.FieldDescriptor(name='w', full_name='EA.Sims4.Network.Quaternion.w', index=3, number=4, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None)], extensions=[], nested_types=[], enum_types=[], options=None, is_extendable=False, extension_ranges=[], serialized_start=109, serialized_end=165)
_TRANSFORM = descriptor.Descriptor(name='Transform', full_name='EA.Sims4.Network.Transform', filename=None, file=DESCRIPTOR, containing_type=None, fields=[descriptor.FieldDescriptor(name='translation', full_name='EA.Sims4.Network.Transform.translation', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None), descriptor.FieldDescriptor(name='orientation', full_name='EA.Sims4.Network.Transform.orientation', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None)], extensions=[], nested_types=[], enum_types=[], options=None, is_extendable=False, extension_ranges=[], serialized_start=167, serialized_end=277)
_LINEARCURVE_CURVEPOINT = descriptor.Descriptor(name='CurvePoint', full_name='EA.Sims4.Network.LinearCurve.CurvePoint', filename=None, file=DESCRIPTOR, containing_type=None, fields=[descriptor.FieldDescriptor(name='x', full_name='EA.Sims4.Network.LinearCurve.CurvePoint.x', index=0, number=1, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None), descriptor.FieldDescriptor(name='y', full_name='EA.Sims4.Network.LinearCurve.CurvePoint.y', index=1, number=2, type=2, cpp_type=6, label=2, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None)], extensions=[], nested_types=[], enum_types=[], options=None, is_extendable=False, extension_ranges=[], serialized_start=352, serialized_end=386)
_LINEARCURVE = descriptor.Descriptor(name='LinearCurve', full_name='EA.Sims4.Network.LinearCurve', filename=None, file=DESCRIPTOR, containing_type=None, fields=[descriptor.FieldDescriptor(name='points', full_name='EA.Sims4.Network.LinearCurve.points', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, options=None)], extensions=[], nested_types=[_LINEARCURVE_CURVEPOINT], enum_types=[], options=None, is_extendable=False, extension_ranges=[], serialized_start=279, serialized_end=386)
_TRANSFORM.fields_by_name['translation'].message_type = _VECTOR3
_TRANSFORM.fields_by_name['orientation'].message_type = _QUATERNION
_LINEARCURVE_CURVEPOINT.containing_type = _LINEARCURVE
_LINEARCURVE.fields_by_name['points'].message_type = _LINEARCURVE_CURVEPOINT
DESCRIPTOR.message_types_by_name['Vector2'] = _VECTOR2
DESCRIPTOR.message_types_by_name['Vector3'] = _VECTOR3
DESCRIPTOR.message_types_by_name['Quaternion'] = _QUATERNION
DESCRIPTOR.message_types_by_name['Transform'] = _TRANSFORM
DESCRIPTOR.message_types_by_name['LinearCurve'] = _LINEARCURVE

class Vector2(message.Message, metaclass=reflection.GeneratedProtocolMessageType):
    DESCRIPTOR = _VECTOR2


class Vector3(message.Message, metaclass=reflection.GeneratedProtocolMessageType):
    DESCRIPTOR = _VECTOR3


class Quaternion(message.Message, metaclass=reflection.GeneratedProtocolMessageType):
    DESCRIPTOR = _QUATERNION


class Transform(message.Message, metaclass=reflection.GeneratedProtocolMessageType):
    DESCRIPTOR = _TRANSFORM


class LinearCurve(message.Message, metaclass=reflection.GeneratedProtocolMessageType):

    class CurvePoint(message.Message, metaclass=reflection.GeneratedProtocolMessageType):
        DESCRIPTOR = _LINEARCURVE_CURVEPOINT

    DESCRIPTOR = _LINEARCURVE

